"""Copyright 2014 Cooper Willis

This file is part of Not Alone.

    Not Alone is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Not Alone is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Not Alone.  If not, see <http://www.gnu.org/licenses/>.

"""

import pygame
from sys import exit
from pygame.locals import *
from lib.vector import *
import math
import random
from lib.collisions import *
import pickle

WIDTH = 640
HEIGHT = 480

SCREEN = pygame.display.set_mode((WIDTH,HEIGHT),0,32)
pygame.display.set_caption("Not Alone")

pygame.init()
pygame.mouse.set_visible(0)
clock = pygame.time.Clock()

def game_start():
	pygame.mixer.music.load('sounds/opening.wav')
	pygame.mixer.music.play(-1,0.0)
	description1 = "Something happened.  You need to find the exit."
	description2 = "There are things in here you want to avoid."
	description3 = "Some of them don't like the light..."

	instructions1 = "Move your person (you'll see their small arc of a flashlight) with WASD."  
	instructions2 = "Press F to turn the spotlight on or off."
	instructions3 = "Give it coordinates with mouse clicks."  
	instructions4 = "Press any key to start, ESC to exit"

	font = pygame.font.SysFont('monospace', 14)
	desc_font1 = font.render(description1,0,(20,180,20))
	desc_font2 = font.render(description2,0,(20,180,20))
	desc_font3 = font.render(description3,0,(20,180,20))
	inst_font1 = font.render(instructions1,0,(20,180,20))
	inst_font2 = font.render(instructions2,0,(20,180,20))
	inst_font3 = font.render(instructions3,0,(20,180,20))
	inst_font4 = font.render(instructions4,0,(20,180,20))
	while True:
		for event in pygame.event.get():
			if event.type == QUIT:
				exit()
			elif event.type == KEYDOWN:
				if event.key == K_ESCAPE:
					exit()
				else:
					main()
		SCREEN.fill((0,0,0))
		SCREEN.blit(desc_font1, (20, 20))
		SCREEN.blit(desc_font2, (20, 60))
		SCREEN.blit(desc_font3, (20, 100))
		SCREEN.blit(inst_font1,(20,180))
		SCREEN.blit(inst_font2,(20,220))
		SCREEN.blit(inst_font3,(20,260))
		SCREEN.blit(inst_font4,(20,300))
		pygame.display.update()

def load_map(map_name):
	F = open('maps/' + map_name + '.pkl','rb')
	map_return = pickle.load(F)
	return map_return

def game_win(time_passed):
	pygame.mixer.music.load('sounds/win.wav')
	pygame.mixer.music.play(-1,0.0)
	font = pygame.font.SysFont('monospace', 12)
	message_font = font.render("Finished in " + str(time_passed),0,(180,0,20))
	play_again = font.render("Play again?  Y/N",0,(180,0,20))
	while True:
		for event in pygame.event.get():
			if event.type == QUIT:
				exit()
			elif event.type == KEYDOWN:
				if event.key == K_y:
					main()
				if event.key == K_n:
					exit()
				if event.key == K_ESCAPE:
					exit()
		SCREEN.fill((0,0,0))
		SCREEN.blit(message_font, (WIDTH/2, HEIGHT/2))
		SCREEN.blit(play_again,(WIDTH/2,HEIGHT/2 + 200))
		pygame.display.update()
		
def game_over():
	pygame.mixer.music.load('sounds/gameover.wav')
	pygame.mixer.music.play(-1,0.0)
	font = pygame.font.SysFont('monospace', 12)
	message_font = font.render("You died." ,0,(180,0,20))
	play_again = font.render("Play again?  Y/N",0,(180,0,20))
	while True:
		for event in pygame.event.get():
			if event.type == QUIT:
				exit()
			elif event.type == KEYDOWN:
				if event.key == K_y:
					main()
				if event.key == K_n:
					exit()
				if event.key == K_ESCAPE:
					exit()
		SCREEN.fill((0,0,0))
		SCREEN.blit(message_font, (WIDTH/2, HEIGHT/2 - 200))
		SCREEN.blit(play_again,(WIDTH/2,HEIGHT/2))
		pygame.display.update()

def main():
	lightdark = pygame.image.load("images/lightdark_t.png").convert_alpha()
	fast = pygame.image.load("images/fast.png").convert_alpha()
	exitim = pygame.image.load("images/exitimage.png").convert_alpha()
	maps = ['map1','map2','map3','map4','map5','map6','map7','map8','map9','map10']
	choice = random.randint(0,len(maps)-1)
	MAP_INFO = load_map(maps[choice])#[CREEPLIST,BGLIST,EXITLIST, PLAYER_START]
	CREEPLIST = MAP_INFO[0]
	BGLIST = MAP_INFO[1]
	EXITLIST = MAP_INFO[2]
	PLAYER_START = MAP_INFO[3]
	for k in PLAYER_START.itervalues():
		PLAYER = k
	font = pygame.font.SysFont('monospace', 12)
	frames = 0
	total_time = 0
	pygame.mixer.music.load('sounds/base1.wav')
	pygame.mixer.music.play(-1,0.0)
	
	SPOTTED1 = pygame.mixer.Sound('sounds/spotted1.wav')
	HIT = pygame.mixer.Sound('sounds/hit.wav')
	HIT2 = pygame.mixer.Sound('sounds/hit2.wav')
	CLICK_DOWN = False
	SPOTLIGHT = Vector(PLAYER.position.x,PLAYER.position.y)
	SPOTLIGHT_SPEED = 5
	SPOTLIGHT_RADIUS = 20
	dest_x = SPOTLIGHT.x
	dest_y = SPOTLIGHT.y
	heading = Vector()
	distance_vector = Vector()
	FLASHLIGHT = 0
	while True:
		clock.tick(30)
		frames += 1
		time_passed_seconds = frames/30.0
		time_passed_seconds
		health_font = font.render(str(PLAYER.health/10),0,(180,0,20))
		for event in pygame.event.get():
			if event.type == QUIT:
				exit()
			elif event.type == MOUSEBUTTONDOWN:
				CLICK_DOWN = True
			elif event.type == MOUSEBUTTONUP:
				CLICK_DOWN = False
				dest_x, dest_y = pygame.mouse.get_pos()
				destination = Vector(dest_x,dest_y)
				heading = Vector.from_points((SPOTLIGHT.x,SPOTLIGHT.y),(dest_x,dest_y))
				heading.normalize()
			elif event.type == KEYDOWN:
				if event.key == K_f:
					FLASHLIGHT = (FLASHLIGHT + 1) %2
				if event.key == K_ESCAPE:
					exit()
		PLAYER.handle_input()
		PLAYER.move()
		if PLAYER.position.x < 0:
			PLAYER.position.x = 0
		if PLAYER.position.y < 0:
			PLAYER.position.y = 0
		if PLAYER.position.x > WIDTH:
			PLAYER.position.x = WIDTH
		if PLAYER.position.y > HEIGHT:
			PLAYER.position.y = HEIGHT
		mouse_x, mouse_y = pygame.mouse.get_pos()
		distance_vector =  Vector.from_points((SPOTLIGHT.x,SPOTLIGHT.y),(dest_x,dest_y))
		if distance_vector.get_magnitude() >= 5 and (FLASHLIGHT == 1):
			SPOTLIGHT += heading*SPOTLIGHT_SPEED
		###CREEP CODE
		for CREEP in CREEPLIST.itervalues():
			creep_radius =  math.sqrt((CREEP.position.x - SPOTLIGHT.x)**2 + (CREEP.position.y - SPOTLIGHT.y)**2)
			if creep_radius < SPOTLIGHT_RADIUS and (FLASHLIGHT == 1) and (not CREEP.aggro):
				if CREEP.heat_up >= 15:
					CREEP.aggro = True
					CREEP.cooldown = 3
					CREEP.time_spotted = pygame.time.get_ticks()
					SPOTTED1.play()
					CREEP.heat_up = 0
				else:
					CREEP.heat_up += 1
			elif CREEP.cooldown > 0: #Roundabout way of doing things
				CREEP.time_now = pygame.time.get_ticks()
				creep_passed = CREEP.time_now - CREEP.time_spotted
				if creep_passed/1000.0 > CREEP.cooldown:
					CREEP.cooldown = 0
					CREEP.aggro = False
			else:
				CREEP.aggro = False

			if CREEP.aggro:
				CREEP.position.x += random.randint(-10,10)
				CREEP.position.y += random.randint(-10,10)
			
			if CREEP.position.x < 0:
				CREEP.position.x = 0
			if CREEP.position.y < 0:
				CREEP.position.y = 0
			if CREEP.position.x > WIDTH:
				CREEP.position.x = WIDTH
			if CREEP.position.y > HEIGHT:
				CREEP.position.y = HEIGHT
				
			if check_creeps(PLAYER,CREEP,15):
				PLAYER.health -= 5
				HIT.play()

		###	
		for BIGCREEP in BGLIST.itervalues():
			if check_creeps(PLAYER,BIGCREEP,40):
				PLAYER.health -= 15
				HIT2.play()
		for EXIT in EXITLIST.itervalues():
			if check_creeps(PLAYER,EXIT,20):
				EXIT.status = True
			if EXIT.status:
				game_win(time_passed_seconds)
		if PLAYER.health <=0:
			game_over()
		###
		#Drawing code, draw the white flashlight stuff before the black monster stuff for stencil effect
		SCREEN.fill((0,0,0))
		if FLASHLIGHT == 1:
			pygame.draw.circle(SCREEN,(255,255,255), (int(SPOTLIGHT.x),int(SPOTLIGHT.y)),SPOTLIGHT_RADIUS,0)
		pygame.draw.circle(SCREEN,(100,100,100),(int(PLAYER.position.x),int(PLAYER.position.y)),5,0)
		player_to_spotlight = math.sqrt((PLAYER.position.x - SPOTLIGHT.x)**2 + (PLAYER.position.y - SPOTLIGHT.y)**2)
		if player_to_spotlight < (SPOTLIGHT_RADIUS + 20) and FLASHLIGHT == 1:
			pygame.draw.arc(SCREEN,(200,200,200),(int(PLAYER.position.x)-20,int(PLAYER.position.y) - 20, 40, 40),(PLAYER.angle*math.pi/180.0 + 45*math.pi/180.0),(PLAYER.angle*math.pi/180.0 + 135*math.pi/180.0),10)
		else:
			pygame.draw.arc(SCREEN,(255,255,255),(int(PLAYER.position.x)-20,int(PLAYER.position.y) - 20, 40, 40),(PLAYER.angle*math.pi/180.0 + 45*math.pi/180.0),(PLAYER.angle*math.pi/180.0 + 135*math.pi/180.0),10)
		for CREEP in CREEPLIST.itervalues():
			SCREEN.blit(fast,(CREEP.position.x - CREEP.width/2,CREEP.position.y - CREEP.height/2))
			#pygame.draw.circle(SCREEN,(0,0,200),(CREEP.position.x,CREEP.position.y),5,0) #FOR DEBUG/DEV ONLY
		for BIGCREEP in BGLIST.itervalues():
			SCREEN.blit(lightdark,(BIGCREEP.position.x - BIGCREEP.width/2,BIGCREEP.position.y - BIGCREEP.height/2))
		for EXIT in EXITLIST.itervalues():
			SCREEN.blit(exitim,(EXIT.position.x - EXIT.width/2,EXIT.position.y - EXIT.height/2))
			#pygame.draw.circle(SCREEN,(0,255,0),(EXIT.position.x,EXIT.position.y),5,0)#FOR DEBUG/DEV ONLY
		if CLICK_DOWN:
			pygame.draw.circle(SCREEN, (200,0,0), (mouse_x, mouse_y),10,0)
		else:
			cursor_to_spotlight = math.sqrt((mouse_x - SPOTLIGHT.x)**2 + (mouse_y - SPOTLIGHT.y)**2)
			if cursor_to_spotlight > SPOTLIGHT_RADIUS:
				pygame.draw.circle(SCREEN, (255,255,255), (mouse_x,mouse_y),5,0)
			else:
				pygame.draw.circle(SCREEN, (0,0,0), (mouse_x,mouse_y),5,0)
		SCREEN.blit(health_font, (20,20))
		pygame.display.update()
		
game_start()