"""Copyright 2014 Cooper Willis

This file is part of Not Alone.

    Not Alone is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Not Alone is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Not Alone.  If not, see <http://www.gnu.org/licenses/>.

"""


import math
import pygame
from pygame.locals import *


#Following the style of Will McGugan's book (www.willmcgugan.com)

class Vector(object):
	
	def __init__(self, x=0, y=0):
		self.x = x
		self.y = y
		
	def __str__(self):
		return "(%s, %s)"%(self.x, self.y)
		
	def __add__(self, other):
		return Vector(self.x + other.x, self.y + other.y)
		
	def __sub__(self, other):
		return Vector(self.x - other.x, self.y - other.y)
		
	def __mul__(self, scalar):
		return Vector(self.x * scalar, self.y * scalar)
		
	def __div__(self, scalar):
		return Vector(self.x - other.x, self.y - other.y)
		
	def get_magnitude(self):
		return math.sqrt(self.x**2 + self.y**2)
		
	def normalize(self):
		mag = self.get_magnitude()
		self.x = self.x/mag
		self.y = self.y/mag
		
	def get_normalized(self):
		mag = self.get_magnitude()
		return Vector(self.x/mag, self.y/mag)
		
	@staticmethod 
	def from_points(p1, p2):
		return Vector( p2[0] - p1[0], p2[1] - p1[1])
		


class Player(object):
	
	def __init__(self, x, y, speed, health,angle=0, movement=0,radius=20):#3,10,0,0,20
		self.position = Vector(x, y)
		self.radius = radius
		self.speed = speed
		self.health = health
		self.angle = angle
		self.direction = Vector(math.sin(self.angle*math.pi/180.0),math.cos(self.angle*math.pi/180.0))
		self.movement = 0
		
	def __str__(self):
		return "%s with speed %s, health %s, angle is %s, direction is %s" %(self.position, self.speed, self.health, self.angle, self.direction)
		
	def handle_input(self):
		keys = pygame.key.get_pressed()
		if keys[K_d]:
			self.angle -=10
		if keys[K_a]:
			self.angle += 10
		if keys[K_w]:
			self.movement = -1
		if keys[K_s]:
			self.movement = 1
		if not keys[K_w] and not keys[K_s]:
			self.movement = 0
		
	def move(self):
		self.direction = Vector(math.sin(self.angle*math.pi/180.0),math.cos(self.angle*math.pi/180.0))
		self.direction.normalize()
		self.position += self.direction*self.speed*self.movement
		
class Creep(object):
	
	def __init__(self, x, y, image,  aggro = False):
		
		self.position = Vector(x,y)
		self.width = image.get_width()
		self.height = image.get_height()
		self.aggro = aggro
		self.cooldown = 0
		self.time_spotted = 0
		self.time_now = 0
		self.heat_up = 0
	def __str__(self):
		return "%s, %s with aggro status %s" %(self.x,self.y, self.aggro)
		
		
class BigCreep(object):
	
	def __init__(self, x, y, image):
		
		self.position = Vector(x,y)
		self.width = image.get_width()
		self.height = image.get_height()

	def __str__(self):
		return "%s, %s " %(self.x,self.y)
		
class Exit(object):
	
	def __init__(self, x, y, image, status = False):
		
		self.position = Vector(x,y)
		self.status = status
		self.width = image.get_width()
		self.height = image.get_height()

		
	def __str__(self):
		return "%s, %s with velocity %s" %(self.x,self.y)
		
		



